package gregapi.worldgen;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.MultiTileEntityRegistry;
import gregapi.code.ArrayListNoNulls;
import gregapi.data.CS.ConfigsGT;
import gregapi.data.MT;
import gregapi.data.OP;
import gregapi.oredict.OreDictManager;
import gregapi.oredict.OreDictMaterial;
import gregapi.util.ST;
import gregapi.util.UT;
import gregapi.util.WD;

import java.util.ArrayList;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

/**
 * @author Gregorius Techneticies
 */
public class Worldgen_GT_Ore_Layer extends WorldgenObject {
	public static ArrayList<Worldgen_GT_Ore_Layer> sList = new ArrayListNoNulls<Worldgen_GT_Ore_Layer>();
	public static int sWeight = 0;
	public final short mMinY, mMaxY, mWeight, mDensity, mSize, mPrimaryMeta, mSecondaryMeta, mBetweenMeta, mSporadicMeta;
	public final boolean mOverworld, mNether, mEnd;
	
	public Worldgen_GT_Ore_Layer(String aName, boolean aDefault, int aMinY, int aMaxY, int aWeight, int aDensity, int aSize, boolean aOverworld, boolean aNether, boolean aEnd, OreDictMaterial aPrimary, OreDictMaterial aSecondary, OreDictMaterial aBetween, OreDictMaterial aSporadic) {
		super(aName, sList, aDefault);
		mOverworld			= ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Overworld"												, aOverworld);
		mNether				= ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Nether"												, aNether);
		mEnd				= ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "TheEnd"												, aEnd);
		mMinY				= (short)					ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "MinHeight"					, aMinY);
		mMaxY				= (short)Math.max(mMinY+5, 	ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "MaxHeight"					, aMaxY));
		mWeight				= (short)					ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "RandomWeight"				, aWeight);
		mDensity			= (short)					ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Density"						, aDensity);
		mSize				= (short)Math.max(1,		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Size"						, aSize));
		mPrimaryMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OrePrimaryLayer"				, aPrimary.mNameInternal)).mID;
		mSecondaryMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OreSecondaryLayer"			, aSecondary.mNameInternal)).mID;
		mBetweenMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OreSporadiclyInbetween"		, aBetween.mNameInternal)).mID;
		mSporadicMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OreSporaticlyAround"			, aSporadic.mNameInternal)).mID;
		
		if (mEnabled) {
			if (UT.Code.exists(mPrimaryMeta		, OreDictMaterial.MATERIAL_ARRAY)) OreDictManager.INSTANCE.triggerVisibility("ore"+OreDictMaterial.MATERIAL_ARRAY[mPrimaryMeta		].mNameInternal);
			if (UT.Code.exists(mSecondaryMeta	, OreDictMaterial.MATERIAL_ARRAY)) OreDictManager.INSTANCE.triggerVisibility("ore"+OreDictMaterial.MATERIAL_ARRAY[mSecondaryMeta	].mNameInternal);
			if (UT.Code.exists(mBetweenMeta		, OreDictMaterial.MATERIAL_ARRAY)) OreDictManager.INSTANCE.triggerVisibility("ore"+OreDictMaterial.MATERIAL_ARRAY[mBetweenMeta		].mNameInternal);
			if (UT.Code.exists(mSporadicMeta	, OreDictMaterial.MATERIAL_ARRAY)) OreDictManager.INSTANCE.triggerVisibility("ore"+OreDictMaterial.MATERIAL_ARRAY[mSporadicMeta		].mNameInternal);
			sWeight += mWeight;
		}
	}
	
	public boolean executeWorldgen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, int aOriginChunkX, int aOriginChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		if (!isGenerationAllowed(aWorld, aDimensionType, (aDimensionType == -1 && mNether) || (aDimensionType == 0 && mOverworld) || (aDimensionType == +1 && mEnd) ? aDimensionType : ~aDimensionType)) return F;
		
		int tMinY = mMinY + aRandom.nextInt(mMaxY - mMinY - 5);
		
		MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry("gt.multitileentity");
		if (tRegistry != null) {
			for (int i = 0, j = 1+aRandom.nextInt(3); i < j; i++) {
				int tX = aChunkX + aRandom.nextInt(16), tZ = aChunkZ + aRandom.nextInt(16);
				for (int tY = Math.min(aWorld.getHeight(), tMinY+25); tY >= tMinY-10 && tY > 0; tY--) {
					Block tContact = aWorld.getBlock(tX, tY, tZ);
					if (tContact.getMaterial().isLiquid()) break;
					if (!tContact.isOpaqueCube()) continue;
					if (tContact.getMaterial() != Material.grass && tContact.getMaterial() != Material.ground && tContact.getMaterial() != Material.sand && tContact.getMaterial() != Material.rock) break;
					if (WD.easyRep(aWorld, tX, tY+1, tZ)) tRegistry.mBlock.placeBlock(aWorld, tX, tY+1, tZ, SIDE_UNKNOWN, (short)32757, aRandom.nextInt(3)!=0?ST.save(UT.NBT.make(), NBT_VALUE, OP.rockGt.mat(UT.Code.select(OreDictMaterial.MATERIAL_ARRAY[mPrimaryMeta], OreDictMaterial.MATERIAL_ARRAY[mPrimaryMeta], OreDictMaterial.MATERIAL_ARRAY[mSecondaryMeta], OreDictMaterial.MATERIAL_ARRAY[mBetweenMeta], OreDictMaterial.MATERIAL_ARRAY[mSporadicMeta]), 1)):aRandom.nextInt(4)!=0&&BIOMES_SPACE.contains(aBiome)?ST.save(UT.NBT.make(), NBT_VALUE, OP.rockGt.mat(MT.MeteoricIron, 1)):UT.NBT.make(), F, T);
					break;
				}
			}
		}
		
		for (int cX=aOriginChunkX-aRandom.nextInt(mSize), eX=aOriginChunkX+16+aRandom.nextInt(mSize), tX=Math.max(aChunkX, cX); tX<=Math.min(aChunkX+15, eX); tX++) {
			for (int cZ=aOriginChunkZ-aRandom.nextInt(mSize), eZ=aOriginChunkZ+16+aRandom.nextInt(mSize), tZ=Math.max(aChunkZ, cZ); tZ<=Math.min(aChunkZ+15, eZ); tZ++) {
				if (mSecondaryMeta >= 0) for (int i=tMinY-1; i<tMinY+2; i++) {
					if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
						Worldgenerator.setOreBlock(aWorld, tX, i, tZ, mSecondaryMeta);
					}
				}
				if (mBetweenMeta >= 0) if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
					Worldgenerator.setOreBlock(aWorld, tX, tMinY+2+aRandom.nextInt(2), tZ, mBetweenMeta);
				}
				if (mPrimaryMeta >= 0) for (int i=tMinY+3; i<tMinY+6; i++) {
					if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
						Worldgenerator.setOreBlock(aWorld, tX, i, tZ, mPrimaryMeta);
					}
				}
				if (mSporadicMeta >= 0) if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
					Worldgenerator.setOreBlock(aWorld, tX, tMinY-1+aRandom.nextInt(7), tZ, mSporadicMeta);
				}
			}
		}
		return T;
	}
}